{% extends 'base/layout.html' %}
{% load buttons %}
{% load static %}
{% load plugins %}
{% load helpers %}

{% block head %}
{% endblock %} 

{% block content %}
<div>
    {% include 'netbox_ipcalculator/core/subnets.html' %}
</div>
{% endblock %}